<?php
declare(strict_types=1);

namespace Smpl\Mydi\Provider;

use Smpl\Mydi\Exception\ContainerException;
use Smpl\Mydi\Exception\NotFoundException;
use Smpl\Mydi\ProviderInterface;

class KeyValue implements ProviderInterface
{
    private $params;

    public function __construct(array $params)
    {
        $this->params = $params;
    }

    public static function fromJSON(string $filePath)
    {
        if (!is_readable($filePath)) {
            throw new ContainerException(
                sprintf(
                    'File: `%s` must be readable',
                    $filePath
                )
            );
        }
        return new self(json_decode(file_get_contents($filePath), true));
    }

    public static function fromPHP(string $filePath)
    {
        if (!is_readable($filePath)) {
            throw new ContainerException(
                sprintf(
                    'File: `%s` must be readable',
                    $filePath
                )
            );
        }
        /** @noinspection PhpIncludeInspection */
        $params = include $filePath;
        if (!is_array($params)) {
            throw new ContainerException(
                sprintf(
                    'File: `%s` must return array',
                    $filePath
                )
            );
        }
        return new self($params);
    }


    public function get(string $name)
    {
        if (!$this->has($name)) {
            throw new NotFoundException($name);
        }
        return $this->params[$name];
    }

    public function has(string $name): bool
    {
        return array_key_exists($name, $this->params) ? true : false;
    }
}