<?php
declare(strict_types=1);

namespace Smpl\Mydi\Test\Provider;

use PHPUnit\Framework\TestCase;
use Smpl\Mydi\Provider\KeyValue;
use Smpl\Mydi\ProviderInterface;

class KeyValueTest extends TestCase
{
    private $exampleDir = __DIR__ . '/../Example';

    public function testHas()
    {
        $keyValue = new KeyValue(['key' => 'value']);
        $this->assertTrue($keyValue->has('key'));
        $this->assertFalse($keyValue->has('invalid key'));
    }

    /**
     * @expectedException \Psr\Container\NotFoundExceptionInterface
     */
    public function testGet()
    {
        $keyValue = new KeyValue(['key' => 'value']);
        $this->assertSame('value', $keyValue->get('key'));
        $keyValue->get('invalid name');
    }

    /**
     * @expectedException \Psr\Container\ContainerExceptionInterface
     * @expectedExceptionMessage File: `invalid.json` must be readable
     */
    public function testFromJSON()
    {
        $keyValue = KeyValue::fromJSON($this->exampleDir . '/config.json');
        $this->assertTrue($keyValue instanceof ProviderInterface);
        $this->assertSame('some string', $keyValue->get('string'));

        KeyValue::fromJSON('invalid.json');
    }

    /**
     * @expectedException \Psr\Container\ContainerExceptionInterface
     * @expectedExceptionMessage File: `invalid.php` must be readable
     */
    public function testFromPHP()
    {
        $keyValue = KeyValue::fromPHP($this->exampleDir . '/config.php');
        $this->assertTrue($keyValue instanceof ProviderInterface);
        $this->assertSame('some string', $keyValue->get('string'));

        KeyValue::fromPHP('invalid.php');
    }

    /**
     * @expectedException \Psr\Container\ContainerExceptionInterface
     * @expectedExceptionMessageRegExp |.*Example/ClassStd.php` must return array|
     */
    public function testFromPHPReturnNotArray()
    {
        KeyValue::fromPHP($this->exampleDir . '/ClassStd.php');
    }
}